


import bpy
import os
bl_info = {
    "name": "Default Import/Export",
    "author": "Amandeep",
    "description": "Automatically apply default operator presets for Import/Export",
    "blender": (2, 90, 0),
    "version": (1, 1, 0),
    "warning": "",
    "category": "Object",
}

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTIBILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
def preferences():
    return bpy.context.preferences.addons[__package__].preferences
class de_operators(bpy.types.PropertyGroup):
    name:bpy.props.StringProperty(default="",name="Label")
    operator:bpy.props.StringProperty(default="",name="Operator")
class QMPrefs(bpy.types.AddonPreferences):
    #extra_imports:bpy.props.StringProperty(default="",name="Extra Imports")
    #extra_exports:bpy.props.StringProperty(default="",name="Extra Exports")
    extra_imports:bpy.props.CollectionProperty(type=de_operators,name="Extra Imports")
    extra_exports:bpy.props.CollectionProperty(type=de_operators,name="Extra Exports")
    
    bl_idname = __package__
    def draw(self,context):
        layout=self.layout
        layout.label(text="Extra Import Operators")
        for ei in self.extra_imports:
            row=layout.row()
            row=row.split(factor=0.9)
            r1=row.row()
            r1.prop(ei,'name')
            r1.prop(ei,'operator')
            row=row.row()
            op=row.operator('die.deleteoperator')
            op.name=ei.name
            op.type="Import"
        layout.operator("die.addextraoperator",text="Add New Import Operator").type='Import'
        layout.separator(factor=1)
        layout.label(text="Extra Export Operators")
        for ee in self.extra_exports:
            row=layout.row()
            row=row.split(factor=0.9)
            r1=row.row()
            r1.prop(ee,'name')
            r1.prop(ee,'operator')
            row=row.row()
            op=row.operator('die.deleteoperator')
            op.name=ee.name
            op.type="Export"
        layout.operator("die.addextraoperator",text="Add New Export Operator").type='Export'
import_ops={'Collada (Default) (.dae)':'wm.collada_import','Alembic(.abc)':'wm.alembic_import','Universal Scene Description (.usd/.usds/.usda)':'wm.usd_import','SVG as Grease Pencil':'wm.gpencil_import_svg','Motion Capture (.bvh)':'import_anim.bvh','Scalable Vector Graphics (.svg)':'import_curve.svg'
,'Stanford (.ply)':'import_mesh.ply','Stl (.stl)':'import_mesh.stl','FBX (.fbx)':'import_scene.fbx','glTF 2.0 (.glb/.gltf)':'import_scene.gltf',
'Wavefront(.obj)':'import_scene.obj','X3D Extensible 3D (.x3d)':'import_scene.x3d'}
export_ops={'Collada (Default) (.dae)':'wm.collada_export','Alembic(.abc)':'wm.alembic_export','Universal Scene Description (.usd/.usds/.usda)':'wm.usd_export','Grease Pencil as SVG':'wm.gpencil_export_svg','Grease Pencil as PDF':'wm.gpencil_export_pdf'
,'Stanford (.ply)':'export_mesh.ply','Stl (.stl)':'export_mesh.stl','FBX (.fbx)':'export_scene.fbx','glTF 2.0 (.glb/.gltf)':'export_scene.gltf',
'Wavefront(.obj)':'export_scene.obj','X3D Extensible 3D (.x3d)':'export_scene.x3d'}
skip_keys=['filepath','directory']
class DIE_OT_Add_Operator(bpy.types.Operator):
    bl_idname = "die.addextraoperator"
    bl_label = "Add Operator"
    bl_description = ""
    bl_options = {"REGISTER", "UNDO"}
    type:bpy.props.StringProperty(default="")

    def execute(self, context):
        if self.type=='Import':
            t=preferences().extra_imports.add()
            
        elif self.type=='Export':
            t=preferences().extra_exports.add()
        else:
            pass
        return {'FINISHED'}
class DIE_OT_Delete_Operator(bpy.types.Operator):
    bl_idname = "die.deleteoperator"
    bl_label = "Delete"
    bl_description = ""
    bl_options = {"REGISTER", "UNDO"}
    type:bpy.props.StringProperty(default="")
    name:bpy.props.StringProperty(default="")
    def execute(self, context):
        if self.type=='Import':
            t=preferences().extra_imports.find(self.name)
            preferences().extra_imports.remove(t)
        elif self.type=='Export':
            t=preferences().extra_exports.find(self.name)
            preferences().extra_exports.remove(t)
        else:
            pass
        return {'FINISHED'}
class DIE_OT_CallOperator(bpy.types.Operator):
    bl_idname = "die.callwithdefault"
    bl_label = "Import/Export"
    bl_description = ""
    bl_options = {"REGISTER", "UNDO"}
    op:bpy.props.StringProperty(default="")

    def execute(self, context):
        presets_dir=os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(__file__))),"presets","operator",self.op)
        useful={}
        props=""
        if os.path.isdir(presets_dir) and 'default.py' in os.listdir(presets_dir):
            with open(os.path.join(presets_dir,"default.py"),mode="r") as f:
                lines=f.readlines()
                for line in lines:
                    if " = " in line and "op." in line:
                        p1=line.split(" = ")[0]
                        if "op." in p1:
                            p2=eval(line.split(" = ")[1].replace("\n",''))
                        
                            useful[p1.replace("op.","")]=p2
            props=""
            i=0
            for key,value in useful.items():
                i+=1
                if i>0:
                    
                    if type(value)==str:
                        if key not in skip_keys:
                                  
                            props=props+","+key+"='"+str(value)+"'"
                    else:
                        props=props+","+key+"="+str(value)
        props=f"{props}"
        #print(props)
        try:
            eval(f"bpy.ops.{self.op}('INVOKE_DEFAULT'"+props+")")
        except Exception as e:
            print(e)    
        return {'FINISHED'}

class DIE_MT_Import(bpy.types.Menu):
    bl_label="Original Imports"
    def draw(self, context):
        import_draw(self, context)
class DIE_MT_Export(bpy.types.Menu):
    bl_label="Original Exports"
    def draw(self, context):
        export_draw(self, context)
def draw_into_import(self, context):
    layout=self.layout
    #layout.menu("DIE_MT_Import")
    base_type = bpy.types.Operator
    ee={}
    extra_exports=[a.operator.replace("bpy.ops.","").replace("()","") for a in preferences().extra_imports]
    #print(extra_exports)
    for typename in dir(bpy.types):
        try:
            bl_type = getattr(bpy.types, typename)
            if issubclass(bl_type, base_type):
                #print(bl_type.)
                if bl_type.bl_idname in extra_exports:
                    
                    ee[get_label_imports(bl_type.bl_idname)]=bl_type.bl_idname        
        except:
                pass
    for key,value in import_ops.items():

            layout.operator("die.callwithdefault",text=key).op=value
    for key,value in ee.items():

            layout.operator("die.callwithdefault",text=key).op=value
    layout.menu("DIE_MT_Import")
def get_label_exports(op):
    label=[a.name for a in preferences().extra_exports if a.operator.replace("bpy.ops.","").replace("()","")==op]
    #print(label)
    if label:
        return label[0]
    
    return "None"
def get_label_imports(op):
    label=[a.name for a in preferences().extra_imports if a.operator.replace("bpy.ops.","").replace("()","")==op]
    if label:
        return label[0]
    return "None"
def draw_into_export(self, context):
    layout=self.layout
    #layout.menu("DIE_MT_Export")
    base_type = bpy.types.Operator
    ee={}
    extra_exports=[a.operator.replace("bpy.ops.","").replace("()","") for a in preferences().extra_exports]
    #print(extra_exports)
    for typename in dir(bpy.types):
        try:
            bl_type = getattr(bpy.types, typename)
            if issubclass(bl_type, base_type):
                if bl_type.bl_idname in extra_exports:
                    ee[get_label_exports(bl_type.bl_idname)]=bl_type.bl_idname        
        except Exception as e:
                pass
    for key,value in export_ops.items():

            layout.operator("die.callwithdefault",text=key).op=value
    for key,value in ee.items():

            layout.operator("die.callwithdefault",text=key).op=value
    layout.menu("DIE_MT_Export")
classes = (de_operators,DIE_OT_Delete_Operator,DIE_OT_Add_Operator,DIE_OT_CallOperator,QMPrefs,DIE_MT_Import,DIE_MT_Export
           )
import_draw=bpy.types.TOPBAR_MT_file_import.draw
export_draw=bpy.types.TOPBAR_MT_file_export.draw
def register():
    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)
    
    bpy.types.TOPBAR_MT_file_import.draw=lambda s,c:s.layout 
    bpy.types.TOPBAR_MT_file_export.draw=lambda s,c:s.layout 
    bpy.types.TOPBAR_MT_file_import.prepend(draw_into_import)
    bpy.types.TOPBAR_MT_file_export.prepend(draw_into_export)


def unregister():

    from bpy.utils import unregister_class

    for cls in reversed(classes):
        unregister_class(cls)
    bpy.types.TOPBAR_MT_file_import.draw=import_draw
    bpy.types.TOPBAR_MT_file_export.draw=export_draw
    bpy.types.TOPBAR_MT_file_import.remove(draw_into_import)
    bpy.types.TOPBAR_MT_file_export.remove(draw_into_export)
if __name__ == "__main__":
    register()
